#ifndef MINES_H_
#define MINES_H_

#include <iostream>
using namespace std;
#include "StdPijo.h"
using namespace STDP;

int const LEFT_MARGIN=0;
int const TOP_MARGIN=1;
int const MAX_WIDTH=80;//80
int const MAX_HEIGHT=21;//24
int const KSHOOT=98;
int const KUP2=119;
int const KLEFT2=97;
int const KRIGHT2=100;
int const KDOWN2=115;
int const KUP3=87;
int const KLEFT3=65;
int const KRIGHT3=68;
int const KDOWN3=83;


class Coordenadas{
public:
	int x, y;
	Coordenadas(){x=0; y=0;};
	int getX()const{return x;};
	int getY()const{return y;};
	void setX(int x1){x=x1;};
	void setY(int y1){y=y1;};
};

class Coin{
public:
	int x, y;
	char sym;
	Coin(){x=0; y=0; sym='c';};
};

class Slider{
public:
	int x, y, length;
	char sym;
	Slider(){x=0; y=0; length=5; sym=16;};
};

class Teleporter{
public:
	int x1, y1, x2, y2;
	char sym;
	Teleporter(){x1=0; y1=0; x2=0; y2=0; sym='t';};
};

class Player{
public:
	int x, y;
	char sym;
	Player(){x=0; y=0; sym='@';};
};

class Ball{
public:
	int x, y;
	char sym;
	Ball(){x=0; y=0; sym='.';};
};

class Mine{
public:
	int x, y;
	char sym;
	Mine(){x=0; y=0; sym='*';};
};

class ReservedArea{
public:
	int x1, y1, x2, y2;
	bool horizontal;
	string text1;
	string text2;
	ReservedArea(){x1=1; y1=1; x2=0; y2=0; horizontal=0; text1="S T A R T"; text2="F I N I S H";};
};

class Stage{
private:
	int width;
	int height;
	Mine mine[500];
	Slider slider[10];
	Teleporter teleporter[10];
	Coin coin[500];
	Player player;
	Ball ball;
	ReservedArea reservedAreas;
	int nSliders;
	int nTeleporters;
	int nMines;
	int nBalls;
	char board[MAX_WIDTH-LEFT_MARGIN][MAX_HEIGHT-TOP_MARGIN];
	char mov[MAX_WIDTH][MAX_HEIGHT];
	int movements;
	int nCoins;
	int coins;
	int points;
	bool horizontal;

public:
	Stage(int level);
	void clearBoard();
	void clearMovements();
	//void showStage();
	void putReservedAreas();
	void putSliders();
	void putTeleporters();
	void putMines();
	void putCoins();
	void putPlayer();
	void showReservedAreas(StdPijo &stdp);
	void show(StdPijo &stdp)const;
	void showCoins(StdPijo &stdp);
	void showSliders(StdPijo &stdp);
	void showTeleporters(StdPijo &stdp);
	void showMines(StdPijo &stdp, bool kind);
	void showPlayer(StdPijo &stdp);
	void showMovements(StdPijo &stdp);
	void showBall(StdPijo &stdp);
	void deleteMine(StdPijo &stdp, int numMine);
	void hideBall(StdPijo &stdp);
	bool endBall();
	int explosionBall();
	void showTexts(StdPijo &stdp, int level, int scores, int time);
	bool isMovement(int key);
	int countMines();
	void move(int key, StdPijo &stdp);
	bool isExplosion();
	void showExplosion(StdPijo &stdp);
	bool isWin();
	void showWin(StdPijo &stdp);
	bool isMoney();
	void addCoin();
	int getPoints();
	void hideCoin(StdPijo &stdp);
	bool isShoot(int key);
	void shoot(StdPijo &stdp, int key);
	void efectos(StdPijo &stdp);
	void addMovement(){movements++;};
	int showScore(StdPijo &stdp, int level, int score, int bonus);

};

class Menu{
public:
	int active;
	int options;
	Menu(int option){active=1; options=option;};
	int show(StdPijo &stdp);
	void selectFormat(StdPijo &stdp, int option);
	void titleFormat(StdPijo &stdp);
	void activeFormat(StdPijo &stdp);
	void normalFormat(StdPijo &stdp);
};

class StageMaker{
public:
	Coordenadas cursor;
	int height;
	int width;
	StageMaker(){cursor.x=1; cursor.y=2; height=20; width=80;};
	bool isDraw(int key);
	void draw(char board[][20], int key);
	void showBoard(StdPijo &stdp, char board[][20]);
	void showCursor(StdPijo &stdp);
	char getSym(char sym);
	void symFormat(StdPijo &stdp, char sym);
	bool isMovement(int key);
	void movement(int key);
	void clearBoard(char board[][20]);
};

class Level{
private:
	int level;
	//Tools tools;
public:
	Level(int level){this->level=level;};
	void selectFormat(StdPijo &stdp, int option);
	void titleFormat(StdPijo &stdp);
	void activeFormat(StdPijo &stdp);
	void normalFormat(StdPijo &stdp);
	void showTitle(StdPijo &stdp);
	void showDescription(StdPijo &stdp);
	void showBenefits(StdPijo &stdp);
	void showSize(StdPijo &stdp);
	void showWage(StdPijo &stdp);
	void showNMines(StdPijo &stdp);
	void showBonus(StdPijo &stdp);
	void showNCoins(StdPijo &stdp);
	void showNBalls(StdPijo &stdp);
	void showNTeleporters(StdPijo &stdp);
	void showNSliders(StdPijo &stdp);
	void showPriceCoin(StdPijo &stdp);
	void showPressAnyKey(StdPijo &stdp);


};

class Game{
private:
	int level;
	int score;
	int records[10][5];
public:
	Game(){level=1; score=0;};
	//void loadGame();
	int showMenu(StdPijo &stdp);
	void showAbility(StdPijo &stdp);
	void showLevel(StdPijo &stdp);
	bool showStage(StdPijo &stdp);
	void showScore(StdPijo &stdp);
	void showControls(StdPijo &stdp);
	void showRanking(StdPijo &stdp);
	void showStageMaker(StdPijo &stdp);
	bool showGameOver(StdPijo &stdp);
	void addLevel();
	void endGame(StdPijo &stdp);

	void cargarRecords();
	int extraerNumero(string cadena, int pos);
	string extraerCadena(string cadena, int pos);
	bool actualizarRecords();//int puntos, int tiempo, int categoria);
	void grabarRecords();
	void resetValues();
};

class Tools{
public:
	int getWidth(int level);
	int getHeight(int level);
	int getNMines(int level);
	bool isHorizontal(int level);
	int getNTeleporters(int level);
	int getNSliders(int level);
	int getNBalls(int level);
	int getNCoins(int level);
	int getWage(int level);
	int getBonus(int level);
	string intToString(int number)const;
	bool isUp(int key);
	bool isDown(int key);
	bool isLeft(int key);
	bool isRight(int key);
	bool isMovement(int key);
	string completeSpaces(int total, int used);
	int intLenght(int num);
	int getDay();
	int getMonth();
	int getYear();

};

#endif

